#ifndef CHROMEOS_PLATFORMS_CAMERA_APP_DOCUMENT_SCANNING_DOCUMENT_SCANNER_H_
#define CHROMEOS_PLATFORMS_CAMERA_APP_DOCUMENT_SCANNING_DOCUMENT_SCANNER_H_

#include <memory>
#include <vector>

extern "C++" {

namespace cvx {
class Mat;
}  // namespace cvx

namespace photos_intel_actions {
class DocumentCornerDetector;
class DocumentQuadProcessor;
}  // namespace photos_intel_actions

namespace chromeos_camera {
namespace document_scanning {

class DocumentScanner {
 public:
  static std::unique_ptr<DocumentScanner> Create(float score_threshold);

  explicit DocumentScanner(float score_threshold);
  virtual ~DocumentScanner();

  // Both data ranges are in [0, 1).
  struct Point {
    float x;
    float y;
  };

  // Detects the corners of a document from given NV12 image.
  virtual bool
  DetectCornersFromNV12Image(const uint8_t* nv12_image,
                             uint32_t width, uint32_t height,
                             std::vector<Point>* corners);

  // Detects the corners of a document from given JPEG image.
  virtual bool
  DetectCornersFromJPEGImage(const uint8_t* jpeg_image,
                             uint32_t image_size,
                             std::vector<Point>* corners);

  // Do post processing on the given document JPEG image based on the given
  // corners.
  virtual bool
  DoPostProcessingFromJPEGImage(const uint8_t* jpeg_image,
                                uint32_t image_size,
                                const std::vector<Point>& corners,
                                std::vector<uint8_t>* processed_jpeg_image);

 private:
  bool GetDocumentCornersFromMat(const cvx::Mat& input_mat,
                                 std::vector<Point>* corners);

  std::unique_ptr<photos_intel_actions::DocumentCornerDetector> detector_;
  std::unique_ptr<photos_intel_actions::DocumentQuadProcessor> quad_processor_;
};

using CreateFn = decltype(&DocumentScanner::Create);

}  // namespace document_scanning
}  // namespace chromeos_camera

}  // extern "C++"

#endif  // CHROMEOS_PLATFORMS_CAMERA_APP_DOCUMENT_SCANNING_DOCUMENT_SCANNER_H_
